@@ -14,10 +14,13 @@
 #include <QObject>
 #include <QScopedPointer>
 #include <QSet>
+#include <QSize>
 #include <QStringList>
 #include <QUrl>
 #include <QVector>
 
+#include <ReviewsBackend/Rating.h>
+
 #include "PackageState.h"
 #include "discovercommon_export.h"
 
@@ -32,16 +35,18 @@ struct Screenshot {
     {
     }
 
-    Screenshot(const QUrl &thumbnail, const QUrl &screenshot, bool isAnimated)
+    Screenshot(const QUrl &thumbnail, const QUrl &screenshot, bool isAnimated, const QSize &thumbnailSize)
         : thumbnail(thumbnail)
         , screenshot(screenshot)
         , isAnimated(isAnimated)
+        , thumbnailSize(thumbnailSize)
     {
     }
 
     QUrl thumbnail;
     QUrl screenshot;
     bool isAnimated = false;
+    QSize thumbnailSize;
 };
 
 using Screenshots = QVector<Screenshot>;
@@ -63,7 +68,6 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
     Q_PROPERTY(bool canExecute READ canExecute CONSTANT)
     Q_PROPERTY(State state READ state NOTIFY stateChanged)
     Q_PROPERTY(QString status READ status NOTIFY stateChanged)
-    Q_PROPERTY(QStringList category READ categories CONSTANT)
     Q_PROPERTY(QUrl homepage READ homepage CONSTANT)
     Q_PROPERTY(QUrl helpURL READ helpURL CONSTANT)
     Q_PROPERTY(QUrl bugURL READ bugURL CONSTANT)
@@ -81,8 +85,8 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
     Q_PROPERTY(QString availableVersion READ availableVersion NOTIFY versionsChanged)
     Q_PROPERTY(QString section READ section CONSTANT)
     Q_PROPERTY(QStringList mimetypes READ mimetypes CONSTANT)
-    Q_PROPERTY(QObject *backend READ backendObject CONSTANT)
-    Q_PROPERTY(QVariant rating READ ratingVariant NOTIFY ratingFetched)
+    Q_PROPERTY(AbstractResourcesBackend *backend READ backend CONSTANT)
+    Q_PROPERTY(Rating rating READ rating NOTIFY ratingFetched)
     Q_PROPERTY(QString appstreamId READ appstreamId CONSTANT)
     Q_PROPERTY(QUrl url READ url CONSTANT)
     Q_PROPERTY(QString executeLabel READ executeLabel CONSTANT)
@@ -92,10 +96,17 @@ class DISCOVERCOMMON_EXPORT AbstractResource : public QObject
     Q_PROPERTY(QString upgradeText READ upgradeText NOTIFY versionsChanged)
     Q_PROPERTY(bool isRemovable READ isRemovable CONSTANT)
     Q_PROPERTY(QString versionString READ versionString NOTIFY versionsChanged)
-    Q_PROPERTY(QString contentRatingText READ contentRatingText CONSTANT)
     Q_PROPERTY(QString contentRatingDescription READ contentRatingDescription CONSTANT)
-    Q_PROPERTY(ContentIntensity contentRatingIntensity READ contentRatingIntensity CONSTANT)
     Q_PROPERTY(uint contentRatingMinimumAge READ contentRatingMinimumAge CONSTANT)
+    Q_PROPERTY(QStringList topObjects READ topObjects CONSTANT)
+    Q_PROPERTY(QStringList bottomObjects READ bottomObjects CONSTANT)
+    Q_PROPERTY(QString verifiedMessage READ verifiedMessage CONSTANT)
+    Q_PROPERTY(QString verifiedIconName READ verifiedIconName CONSTANT)
+    Q_PROPERTY(Type type READ type CONSTANT)
+
+    // Resolve circular dependency for QObject* properties in both classes
+    Q_MOC_INCLUDE("resources/AbstractResourcesBackend.h")
+
 public:
     /**
      * This describes the state of the resource
@@ -120,12 +131,6 @@ public:
     };
     Q_ENUM(State)
 
-    enum ContentIntensity {
-        Mild,
-        Intense,
-    };
-    Q_ENUM(ContentIntensity)
-
     /**
      * Constructs the AbstractResource with its corresponding backend
      */
@@ -153,7 +158,7 @@ public:
 
     virtual State state() = 0;
 
-    virtual QStringList categories() = 0;
+    virtual bool hasCategory(const QString &category) const = 0;
     ///@returns a URL that points to the app's website
     virtual QUrl homepage();
     ///@returns a URL that points to the app's online documentation
@@ -168,7 +173,14 @@ public:
     enum Type {
         Application,
         Addon,
-        Technical,
+        /**
+         * This is where Flatpak runtimes would go
+         */
+        ApplicationSupport,
+        /**
+         * This is where system updates would go
+         */
+        System,
     };
     Q_ENUM(Type)
     virtual Type type() const = 0;
@@ -197,8 +209,8 @@ public:
 
     virtual QString appstreamId() const;
 
-    void addMetadata(const QString &key, const QJsonValue &value);
-    QJsonValue getMetadata(const QString &key);
+    void addMetadata(const QLatin1StringView &key, const QJsonValue &value);
+    QJsonValue getMetadata(const QLatin1StringView &key);
 
     bool canUpgrade();
     bool isInstalled();
@@ -208,7 +220,6 @@ public:
     virtual QString status();
 
     AbstractResourcesBackend *backend() const;
-    QObject *backendObject() const;
 
     /**
      * @returns a name sort key for faster sorting
@@ -220,8 +231,7 @@ public:
      *
      * @returns the rating for the resource or null if not available
      */
-    Rating *rating() const;
-    QVariant ratingVariant() const;
+    virtual Rating rating() const;
 
     bool categoryMatches(Category *cat);
 
@@ -256,11 +266,45 @@ public:
 
     virtual QString versionString();
 
-    virtual QString contentRatingText() const;
-    virtual ContentIntensity contentRatingIntensity() const;
     virtual QString contentRatingDescription() const;
     virtual uint contentRatingMinimumAge() const;
 
+    /**
+     * @returns List of component URLs to display at the top of application page.
+     */
+    virtual QStringList topObjects() const;
+    /**
+     * @returns List of component URLs to display at the bottom of application page.
+     */
+    virtual QStringList bottomObjects() const;
+    [[nodiscard]] virtual bool hasResolvedIcon() const;
+    virtual void resolveIcon();
+
+    /**
+     * @returns whether the verification message of the resource
+     *
+     * Normally app stores will have extra trust checks on certain assets.
+     * If the string is empty it means that the asset isn't verified.
+     * The text will contain the explanation of what the verification entails.
+     */
+    virtual QString verifiedMessage() const
+    {
+        return {};
+    }
+
+    /**
+     * @returns the icon name of the verification badge appropriate to the verification
+     */
+    virtual QString verifiedIconName() const
+    {
+        return QStringLiteral("checkmark");
+    }
+
+    virtual bool updateNeedsAttention()
+    {
+        return false;
+    }
+
 public Q_SLOTS:
     virtual void fetchScreenshots();
     virtual void fetchChangelog() = 0;
